#pragma once
//////////////////////////////////////////////////////////////////////
// geometryPositionNormalTexture.h: interface for the Geometry class.
// -Kyoung Shin Park (2014 Spring)
//////////////////////////////////////////////////////////////////////
#include "CatmullRomCurveAnimation.h"
#include "vertexBufferObject.h"


namespace moglclass
{

class VertexBufferObject;
class KeyFrame;
class CatmullRomCurveAnimation;

class GeometryPositionNormalTexture 
{
public:
	GeometryPositionNormalTexture();
	virtual ~GeometryPositionNormalTexture() { std::cout << "~Geometry" << std::endl; };
	virtual void init() = 0;
    virtual void draw() = 0;
	virtual bool update(float elapsedTime);
	virtual void createVAO();
	//const float* getColor() { return &color[0]; };
	//glm::vec3 Color() { return color; };
	void setPosition(glm::vec3 p_) { p = p_; init(); };

	// animation
	void setAnimation(glm::vec3 start_, glm::vec3 end_, float duration_);
	void setKeyframeAnimation(std::vector<KeyFrame> frames_);
	void activate() { active = true; }
	void deactivate() { active = false; }
	bool isAnimationDone() { return done; }
	bool isActive() { return active; }
	void reset() { 	active = false; done = false; }

protected:
	//glm::vec3 color;
	glm::vec3 p; // center position

	// vao & vbo
	int numVertices;
	GLuint vao;
    VertexBufferObject vbo;
	bool isLoaded;

	// animation
	bool done;
	bool active;
	glm::vec3 start;
	glm::vec3 end;
	float duration;
	CatmullRomCurveAnimation * curve;
};

class Triangle : public GeometryPositionNormalTexture
{
public:
    Triangle(glm::vec3 p=glm::vec3(0, 0, 0), glm::vec3 v1=glm::vec3(-1, -1, 0), glm::vec3 v2=glm::vec3(1, -1, 0), glm::vec3 v3=glm::vec3(0, 1, 0));
	void set(glm::vec3 p_, glm::vec3 v1_, glm::vec3 v2_, glm::vec3 v3_);
	void init();
	void draw();
	
private:
	glm::vec3 v1;
	glm::vec3 v2;
	glm::vec3 v3;
};


class Circle : public GeometryPositionNormalTexture
{
public:
	Circle(glm::vec3 p_=glm::vec3(0, 0, 0), float radius=1.0f, int slices=16);
	void set(glm::vec3 p_, float radius_, int slices_);
	void init();
	void draw();

private:
	float radius;
	int slices;
};

class Quad : public GeometryPositionNormalTexture
{
public:
	Quad(glm::vec3 p=glm::vec3(0, 0, 0), glm::vec3 n=glm::vec3(0, 0, 1), glm::vec3 u=glm::vec3(0, 1, 0), float width=1.0f, float height=1.0f);
	void set(glm::vec3 p=glm::vec3(0, 0, 0), glm::vec3 n=glm::vec3(0, 0, 1), glm::vec3 u=glm::vec3(0, 1, 0), float width=1.0f, float height=1.0f);
	void init();
	void draw();

private:
	glm::vec3 p; // position
	glm::vec3 n; // normal vector
	glm::vec3 u; // up vector
	float width;
	float height;
};

class Grid : public GeometryPositionNormalTexture
{
public:
	Grid(glm::vec3 p = glm::vec3(0, 0, 0), glm::vec3 n = glm::vec3(0, 0, 1), glm::vec3 u = glm::vec3(0, 1, 0), float width = 1.0f, float height = 1.0f, int rows = 1, int columns = 1);
	void set(glm::vec3 p_, glm::vec3 n_, glm::vec3 u_, float width_, float height_, int rows = 1, int columns = 1);
	void init();
	void draw();

private:
	glm::vec3 n; // normal vector
	glm::vec3 u; // up vector
	float width;
	float height;
	int rows;
	int columns;
	float tileWidth;
	float tileHeight;
};

class Sphere : public GeometryPositionNormalTexture
{
public:
	Sphere(glm::vec3 p_=glm::vec3(0, 0, 0), float radius=1.0f, int slices=16, int stacks=16);
	void set(glm::vec3 p_, float radius_, int slices_, int stacks_);
	void init();
	void draw();

private:
	float radius;
	int slices;
	int stacks;
};

class Cylinder : public GeometryPositionNormalTexture
{
public:
	Cylinder(glm::vec3 p_=glm::vec3(0, 0, 0), float radius=1.0f, float height=1.0f, int slices=16);
	void set(glm::vec3 p_, float radius_, float height_, int slices_);
	void init();
	void draw();

private:
	float radius;
	float height;
	int slices;
};

class Cone : public GeometryPositionNormalTexture
{
public:
	Cone(glm::vec3 p_ = glm::vec3(0, 0, 0), float radius = 1.0f, float height = 1.0f, int slices = 16);
	void set(glm::vec3 p_, float radius_, float height_, int slices_);
	void init();
	void draw();

private:
	float radius;
	float height;
	int slices;
};

class Cube : public GeometryPositionNormalTexture
{
public:
	Cube(glm::vec3 p_=glm::vec3(0, 0, 0), float size=1.0f);
	void set(glm::vec3 p_, float size_);
	void init();
	void draw();

private:
	float size;
};

class Torus : public GeometryPositionNormalTexture
{
public:
	Torus(glm::vec3 p_=glm::vec3(0, 0, 0), float radius=1.0f, float tubeRadius=0.25f, int rings=16, int sides=16);
	void set(glm::vec3 p_, float radius_, float tubeRadius_, int rings_, int sides_);
	void init();
	void draw();

private:
	float radius;
	float tubeRadius;
	int rings;
	int sides;
};

class Parallelepiped : public GeometryPositionNormalTexture
{
public:
	Parallelepiped(glm::vec3 p=glm::vec3(0, 0, 0), glm::vec3 u=glm::vec3(1, 0, 0), glm::vec3 v=glm::vec3(0, 0, 1), glm::vec3 w=glm::vec3(0, 1, 0));
	void set(glm::vec3 p_, glm::vec3 u_, glm::vec3 v_, glm::vec3 w_);
	void init();
	void draw();

private:
	//glm::vec3 p;
	glm::vec3 u;
	glm::vec3 v;
	glm::vec3 w;
};

class Sa 
{
public:
	Sa();
	void init();
	void draw();

private:
	Parallelepiped p[4];
};

class MultitextureQuad : public Quad
{
public:
	MultitextureQuad(glm::vec3 p = glm::vec3(0, 0, 0), glm::vec3 n = glm::vec3(0, 0, 1), glm::vec3 u = glm::vec3(0, 1, 0), float width = 1.0f, float height = 1.0f);
	void set(glm::vec3 p = glm::vec3(0, 0, 0), glm::vec3 n = glm::vec3(0, 0, 1), glm::vec3 u = glm::vec3(0, 1, 0), float width = 1.0f, float height = 1.0f);
	void init();
	void draw();
};

} //namespace